@@ -138,10 +138,13 @@ axutil_date_time_t* encodeDateTime(const time_t& ts) {
 
 void mapFieldsToSummary(const JobSummaryFields& fields, JobSummary* _summary) {
 
+	JobServerObject* jso = JobServerObject::getInstance();
 	// JobID should already been in our summary
 	SubmissionID* sid = new SubmissionID;
 	sid->setName(fields.submission_id);
 	sid->setOwner(fields.owner);
+	sid->setPool(jso->getPool());
+	sid->setScheduler(jso->getName());
 	_summary->getId()->setSubmission(sid);
 	// do date/time conversion
 	_summary->setQueued(encodeDateTime(fields.queued));
@@ -255,6 +258,7 @@ GetSubmissionSummaryResponse* AviaryQueryServiceSkeleton::getSubmissionSummary(w
 	}
 
 	for (SubmissionCollectionType::iterator i = sub_map.begin(); sub_map.end() != i; i++) {
+		JobServerObject* jso = JobServerObject::getInstance();
 		SubmissionSummary* summary = new SubmissionSummary;
 		SubmissionObject *submission = (*i).second;
 
@@ -263,6 +267,8 @@ GetSubmissionSummaryResponse* AviaryQueryServiceSkeleton::getSubmissionSummary(w
 			sid->setName(submission->getName());
 			sid->setOwner(submission->getOwner());
 			sid->setQdate(submission->getOldest());
+			sid->setPool(jso->getPool());
+			sid->setScheduler(jso->getName());
 			summary->setId(sid);
 			summary->setCompleted(submission->getCompleted().size());
 			summary->setHeld(submission->getHeld().size());
@@ -504,17 +510,41 @@ GetJobDataResponse* AviaryQueryServiceSkeleton::getJobData(wso2wsf::MessageConte
 }
 
 SubmissionID* makeSubmissionID(SubmissionObject* obj) {
+  JobServerObject* jso = JobServerObject::getInstance();
   SubmissionID* sub_id = new SubmissionID;
   sub_id->setName(obj->getName());
   sub_id->setOwner(obj->getOwner());
   sub_id->setQdate(obj->getOldest());
+  sub_id->setPool(jso->getPool());
+  sub_id->setScheduler(jso->getName());
   return sub_id;
 }
 
-bool qdateCompare(SubmissionIndexType::value_type& x, SubmissionIndexType::value_type& y) {
+bool qdateCompare(SubmissionMultiIndexType::value_type& x, SubmissionMultiIndexType::value_type& y) {
   return x.first <= y.first;
 }
 
+// need a way to step over qdate dupes when a specific offset comes in
+// treat name/owner as the secondary "key"
+bool advanceQdateIndex(SubmissionObject* index, SubmissionID* offset) {
+    bool advance = false;
+
+    // quick check
+    if (index->getOldest() != offset->getQdate()) {
+        return advance;
+    }
+
+    // now see how specific the offset is
+    if (!offset->isNameNil() && !offset->getName().empty()) {
+        advance = (index->getName() == offset->getName());
+    }
+    if (!offset->isOwnerNil() && !offset->getOwner().empty()) {
+        advance = (index->getOwner() == offset->getOwner());
+    }
+
+    return advance;
+}
+
 GetSubmissionIDResponse* AviaryQueryServiceSkeleton::getSubmissionID(wso2wsf::MessageContext* /*outCtx*/ ,GetSubmissionID* _getSubmissionID)
 {
     GetSubmissionIDResponse* response = new GetSubmissionIDResponse;
@@ -522,10 +552,18 @@ GetSubmissionIDResponse* AviaryQueryServiceSkeleton::getSubmissionID(wso2wsf::Me
     ScanMode* mode = NULL;
     
     int size = _getSubmissionID->getSize();
+    int qdate;
     bool scan_back = false;
 
+    // some fast track stuff... should be empty together
+    if (g_qdate_submissions.empty() && g_submissions.empty()) {
+            response->setRemaining(0);
+            return response;
+    }
+
     if (!_getSubmissionID->isOffsetNil()) {
         offset = _getSubmissionID->getOffset();
+        qdate = offset->getQdate();
     }
     
     if (!_getSubmissionID->isModeNil()) {
@@ -533,12 +571,14 @@ GetSubmissionIDResponse* AviaryQueryServiceSkeleton::getSubmissionID(wso2wsf::Me
     }
 
     // see if we are scanning using a qdate index
-    if (!_getSubmissionID->isModeNil()) {
-        SubmissionIndexType::iterator it, start, last;
+    if (mode) {
+
+        SubmissionMultiIndexType::iterator it, start, last;
         int i=0;
 
         scan_back = mode->getScanModeEnum() == ScanMode_BEFORE;
 
+        // BEFORE mode
         if (scan_back) {
             if (offset) {
                 start = max_element(
@@ -550,17 +590,22 @@ GetSubmissionIDResponse* AviaryQueryServiceSkeleton::getSubmissionID(wso2wsf::Me
                         );
             }
             else {
-                start = g_qdate_submissions.end();
+                start = --g_qdate_submissions.end();
             }
-            it=start;
-            do {
-                response->addIds(makeSubmissionID((*it).second));
-                i++;
-                last = it;
+            it=last=start;
+            if (qdate>=(*it).second->getOldest() && qdate>0)  {
+                do {
+                    if (!advanceQdateIndex((*it).second,offset)) {
+                        response->addIds(makeSubmissionID((*it).second));
+                        i++;
+                    }
+                    last = it;
+                }
+                while (g_qdate_submissions.begin()!=it-- && i<size);
             }
-            while (g_qdate_submissions.begin()!=it-- && i<size);
             response->setRemaining(distance(g_qdate_submissions.begin(),last));
         }
+        // AFTER mode
         else {
             if (offset) {
                 start = g_qdate_submissions.upper_bound(offset->getQdate());
@@ -568,11 +613,17 @@ GetSubmissionIDResponse* AviaryQueryServiceSkeleton::getSubmissionID(wso2wsf::Me
             else {
                 start = g_qdate_submissions.begin();
             }
-            for (it=start; it!=g_qdate_submissions.end() && i<size; it++) {
-                response->addIds(makeSubmissionID((*it).second));
-                i++;
+            it = --g_qdate_submissions.end();
+            // TODO: integer rollover, but interop of xsd unsignedInt?
+            if (qdate<it->second->getOldest() && qdate<INT_MAX)  {
+                for (it=start; it!=g_qdate_submissions.end() && i<size; it++) {
+                     if (!advanceQdateIndex((*it).second,offset)) {
+                        response->addIds(makeSubmissionID((*it).second));
+                        i++;
+                     }
+                }
             }
-            response->setRemaining(distance(it,g_qdate_submissions.end()));
+            response->setRemaining(i?distance(it,g_qdate_submissions.end()):0);
         }
 
         return response;
