@@ -148,13 +148,17 @@ FindLastRescueDagNum( const char *primaryDagFile, bool multiDags,
 			int maxRescueDagNum )
 {
 	int lastRescue = 0;
-	bool done = false;
 
 	for ( int test = 1; test <= maxRescueDagNum; test++ ) {
 		MyString testName = RescueDagName( primaryDagFile, multiDags,
 					test );
 		if ( access( testName.Value(), F_OK ) == 0 ) {
 			if ( test > lastRescue + 1 ) {
+					// This should probably be a fatal error if
+					// DAGMAN_USE_STRICT is set, but I'm avoiding
+					// that for now because the fact that this code
+					// is used in both condor_dagman and condor_submit_dag
+					// makes that harder to implement. wenger 2011-01-28
 				dprintf( D_ALWAYS, "Warning: found rescue DAG "
 							"number %d, but not rescue DAG number %d\n",
 							test, test - 1);
@@ -167,7 +171,6 @@ FindLastRescueDagNum( const char *primaryDagFile, bool multiDags,
 		dprintf( D_ALWAYS,
 					"Warning: FindLastRescueDagNum() hit maximum "
 					"rescue DAG number: %d\n", maxRescueDagNum );
-		done = true;
 	}
 
 	return lastRescue;
@@ -219,3 +222,12 @@ RenameRescueDagsAfter(const char *primaryDagFile, bool multiDags,
 		}
 	}
 }
+
+//-------------------------------------------------------------------------
+MyString
+HaltFileName( const MyString &primaryDagFile )
+{
+	MyString haltFile = primaryDagFile + ".halt";
+
+	return haltFile;
+}
