@@ -259,12 +259,12 @@ ToolDaemonProc::StartJob()
 	//
 	FamilyInfo fi;
 	fi.max_snapshot_interval = 15;
-	char const *dedicated_account = NULL;
+	m_dedicated_account = NULL;
 	if (job_universe != CONDOR_UNIVERSE_LOCAL) {
-		dedicated_account = Starter->jic->getExecuteAccountIsDedicated();
+		m_dedicated_account = Starter->jic->getExecuteAccountIsDedicated();
 	}
-	if (dedicated_account) {
-		fi.login = dedicated_account;
+	if (m_dedicated_account) {
+		fi.login = m_dedicated_account;
 		dprintf(D_FULLDEBUG,
 		        "Tracking process family by login \"%s\"\n",
 		        fi.login);
